@@ -276,34 +276,26 @@ class RuleTideman(Rule):
     @cached_property
     def w_(self):
         self.mylog("Compute w", 1)
-        candidates_alive = np.array(range(self.profile_.n_c))
-        preferences_borda_rk = self.profile_.preferences_borda_rk.copy()  # We will put -1 when a candidate loses
-        scores_r = self.profile_.plurality_scores_rk.copy().astype(float)
-        ballots_r = self.profile_.preferences_rk[:, 0].copy()
-        plurality_loser = None
+        plurality_elimination_engine = self.profile_.plurality_elimination_engine()
+        plurality_loser = -1
         while True:
             # Eliminate candidates outside Smith set
+            candidates_alive = plurality_elimination_engine.candidates_alive
             matrix_victories_rk_r = self.profile_.matrix_victories_rk[candidates_alive, :][:, candidates_alive]
             smith_set_r = matrix_victories_to_smith_set(matrix_victories_rk_r)
+            if len(smith_set_r) == 1:
+                return candidates_alive[smith_set_r][0]
             smith_losers = np.delete(candidates_alive, smith_set_r)
-            candidates_alive = candidates_alive[smith_set_r]
-            if len(candidates_alive) == 1:
-                return candidates_alive[0]
             # Eliminate candidate with the lowest Plurality score
-            if plurality_loser is not None:
-                new_ballots = np.argmax(preferences_borda_rk[ballots_r == plurality_loser, :], axis=1)
-                ballots_r[ballots_r == plurality_loser] = new_ballots
-                scores_r += np.bincount(new_ballots, minlength=self.profile_.n_c)
-                scores_r[plurality_loser] = np.nan
+            if plurality_loser != -1:
+                plurality_elimination_engine.update_scores()
             for loser in smith_losers:
-                new_ballots = np.argmax(preferences_borda_rk[ballots_r == loser, :], axis=1)
-                ballots_r[ballots_r == loser] = new_ballots
-                scores_r += np.bincount(new_ballots, minlength=self.profile_.n_c)
-            scores_r[smith_losers] = np.nan
-            plurality_loser = np.where(scores_r == np.nanmin(scores_r))[0][-1]  # Tie-breaking: the last index
-            candidates_alive = np.array([c for c in candidates_alive if c != plurality_loser])
-            if len(candidates_alive) == 1:
-                return candidates_alive[0]
+                plurality_elimination_engine.eliminate_candidate_and_update_scores(loser)
+            scores = plurality_elimination_engine.scores
+            if plurality_elimination_engine.nb_candidates_alive == 2:
+                return np.nanargmax(scores)
+            plurality_loser = np.where(scores == np.nanmin(scores))[0][-1]  # Tie-breaking: the last index
+            plurality_elimination_engine.eliminate_candidate(plurality_loser)
 
     @cached_property
     def scores_(self):
